#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.content import Handler
from imiptools.data import get_address, get_uri, uri_dict, uri_items
from imiptools.handlers.common import CommonFreebusy
from imiptools.profile import Preferences

class PersonHandler(Handler):

    "Handling mechanisms specific to people."

    def _record_and_deliver(self, from_organiser=True, queue=False, cancel=False):

        oa = self.require_organiser_and_attendees(from_organiser)
        if not oa:
            return False

        (organiser, organiser_attr), attendees = organiser_item, attendees = oa

        # Handle notifications and invitations.

        if from_organiser:

            # Process each attendee separately.

            for attendee, attendee_attr in attendees.items():

                if not self.have_new_object(attendee):
                    continue

                # Record other party free/busy information.

                if organiser != attendee:
                    freebusy = self.store.get_freebusy_for_other(attendee, organiser)

                    if organiser_attr.get("PARTSTAT") != "DECLINED":
                        self.update_freebusy_for_other(freebusy, attendee, organiser, self.get_periods())
                    else:
                        self.remove_from_freebusy_for_other(freebusy, attendee, organiser)

                # Store the object and queue any request.

                self.store.set_event(attendee, self.uid, self.obj.to_node())

                if queue:
                    self.store.queue_request(attendee, self.uid)
                elif cancel:
                    self.store.cancel_event(attendee, self.uid)

        # As organiser, update attendance.

        else:
            obj = self.get_object(organiser)

            if obj and self.have_new_object(organiser, obj=obj):

                # Get attendee details in a usable form.

                attendee_map = uri_dict(obj.get_value_map("ATTENDEE"))

                for attendee, attendee_attr in attendees.items():

                    # Update attendance in the loaded object.

                    attendee_map[attendee] = attendee_attr

                    # Record other party free/busy information.

                    if organiser != attendee:
                        freebusy = self.store.get_freebusy_for_other(organiser, attendee)

                        if attendee_attr.get("PARTSTAT") != "DECLINED":
                            self.update_freebusy_for_other(freebusy, organiser, attendee, self.get_periods())
                        else:
                            self.remove_from_freebusy_for_other(freebusy, organiser, attendee)

                # Set the new details and store the object.

                obj["ATTENDEE"] = attendee_map.items()

                self.store.set_event(organiser, self.uid, obj.to_node())

        return True

    def _record_freebusy(self, from_organiser=True):

        "Record free/busy information for the received information."

        freebusy = []

        for value in self.obj.get_values("FREEBUSY") or []:
            if not isinstance(value, list):
                value = [value]
            for v in value:
                try:
                    start, end = v.split("/", 1)
                    freebusy.append((start, end))
                except ValueError:
                    pass

        for sender, sender_attr in uri_items(self.obj.get_items(from_organiser and "ORGANIZER" or "ATTENDEE")):
            self.store.set_freebusy_for_other(get_uri(self.recipient), freebusy, sender)

class Event(PersonHandler):

    "An event handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active event.

        return self.wrap("An addition to an event has been received.", link=False)

    def cancel(self):

        "Queue a cancellation of any active event."

        self._record_and_deliver(from_organiser=True, queue=False, cancel=True)
        return self.wrap("A cancellation has been received.", link=False)

    def counter(self):

        # NOTE: Queue a suggested modification to any active event.

        return self.wrap("A counter proposal has been received.", link=False)

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active event.

        return self.wrap("A declining counter proposal has been received.", link=False)

    def publish(self):

        "Register details of any relevant event."

        self._record_and_deliver(from_organiser=True, queue=False)
        return self.wrap("Details of an event have been received.")

    def refresh(self):

        "Update details of any active event."

        self._record_and_deliver(from_organiser=True, queue=False)
        return self.wrap("An event update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver(from_organiser=False, queue=False)
        return self.wrap("A reply has been received.")

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver(from_organiser=True, queue=True)
        return self.wrap("A request has been received.")

class Freebusy(PersonHandler, CommonFreebusy):

    "A free/busy handler."

    def publish(self):

        "Register free/busy information."

        self._record_freebusy(from_organiser=True)

        # Produce a message if configured to do so.

        preferences = Preferences(get_uri(self.recipient))
        if preferences.get("freebusy_messages") == "notify":
            return self.wrap("A free/busy update has been received.", link=False)

    def reply(self):

        "Record replies and notify the recipient."

        self._record_freebusy(from_organiser=False)

        # Produce a message if configured to do so.

        preferences = Preferences(get_uri(self.recipient))
        if preferences.get("freebusy_messages") == "notify":
            return self.wrap("A reply to a free/busy request has been received.", link=False)

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        # Produce a reply if configured to do so.

        preferences = Preferences(get_uri(self.recipient))
        if preferences.get("freebusy_sharing") == "share":
            return CommonFreebusy.request(self)

class Journal(PersonHandler):

    "A journal entry handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active entry.

        return self.wrap("An addition to a journal entry has been received.", link=False)

    def cancel(self):

        # NOTE: Queue a suggested modification to any active entry.

        return self.wrap("A cancellation has been received.", link=False)

    def publish(self):

        # NOTE: Register details of any relevant entry.

        self._record_and_deliver(from_organiser=True, queue=False)
        return self.wrap("Details of a journal entry have been received.")

class Todo(PersonHandler):

    "A to-do item handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active item.

        return self.wrap("An addition to an item has been received.", link=False)

    def cancel(self):

        # NOTE: Queue a suggested modification to any active item.

        return self.wrap("A cancellation has been received.", link=False)

    def counter(self):

        # NOTE: Queue a suggested modification to any active item.

        return self.wrap("A counter proposal has been received.", link=False)

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active item.

        return self.wrap("A declining counter proposal has been received.", link=False)

    def publish(self):

        "Register details of any relevant item."

        self._record_and_deliver(from_organiser=True, queue=False)
        return self.wrap("Details of an item have been received.")

    def refresh(self):

        "Update details of any active item."

        self._record_and_deliver(from_organiser=True, queue=False)
        return self.wrap("An item update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver(from_organiser=False, queue=False)
        return self.wrap("A reply has been received.")

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver(from_organiser=True, queue=True)
        return self.wrap("A request has been received.")

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
